@@ -17,14 +17,6 @@
   Unit test for 'interface.py'.
 """
 
-# Help with Python 3 compatibility, where the print statement is a function, an
-# implicit relative import is invalid, and the '/' operator performs true
-# division.  Example:  print 'hello world' raises a 'SyntaxError' exception.
-from __future__ import print_function
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import unicode_literals
-
 import os
 import time
 import datetime
@@ -60,16 +52,26 @@ from securesystemslib.formats import (
 from securesystemslib.exceptions import Error, FormatError, CryptoError
 
 from securesystemslib.interface import (
+    _generate_and_write_rsa_keypair,
     generate_and_write_rsa_keypair,
+    generate_and_write_rsa_keypair_with_prompt,
+    generate_and_write_unencrypted_rsa_keypair,
     import_rsa_privatekey_from_file,
     import_rsa_publickey_from_file,
+    _generate_and_write_ed25519_keypair,
     generate_and_write_ed25519_keypair,
+    generate_and_write_ed25519_keypair_with_prompt,
+    generate_and_write_unencrypted_ed25519_keypair,
     import_ed25519_publickey_from_file,
     import_ed25519_privatekey_from_file,
+    _generate_and_write_ecdsa_keypair,
     generate_and_write_ecdsa_keypair,
+    generate_and_write_ecdsa_keypair_with_prompt,
+    generate_and_write_unencrypted_ecdsa_keypair,
     import_ecdsa_publickey_from_file,
     import_ecdsa_privatekey_from_file,
-    import_publickeys_from_file)
+    import_publickeys_from_file,
+    import_privatekey_from_file)
 
 
 
@@ -100,13 +102,12 @@ class TestInterfaceFunctions(unittest.TestCase):
 
 
   def test_rsa(self):
-    """Test RSA key generation and import interface functions. """
+    """Test RSA key _generation and import interface functions. """
 
     # TEST: Generate default keys and import
     # Assert location and format
     fn_default = "default"
-    fn_default_ret = generate_and_write_rsa_keypair(
-        filepath=fn_default, password="")
+    fn_default_ret = _generate_and_write_rsa_keypair(filepath=fn_default)
 
     pub = import_rsa_publickey_from_file(fn_default + ".pub")
     priv = import_rsa_privatekey_from_file(fn_default)
@@ -123,13 +124,13 @@ class TestInterfaceFunctions(unittest.TestCase):
     # Assert importable without password
     fn_empty_prompt = "empty_prompt"
     with mock.patch("securesystemslib.interface.get_password", return_value=""):
-      generate_and_write_rsa_keypair(filepath=fn_empty_prompt)
+      _generate_and_write_rsa_keypair(filepath=fn_empty_prompt, prompt=True)
     import_rsa_privatekey_from_file(fn_empty_prompt)
 
 
     # TEST: Generate keys with auto-filename, i.e. keyid
     # Assert filename is keyid
-    fn_keyid = generate_and_write_rsa_keypair(password="")
+    fn_keyid = _generate_and_write_rsa_keypair()
     pub = import_rsa_publickey_from_file(fn_keyid + ".pub")
     priv = import_rsa_privatekey_from_file(fn_keyid)
     self.assertTrue(
@@ -140,7 +141,7 @@ class TestInterfaceFunctions(unittest.TestCase):
     # Assert length
     bits = 4096
     fn_bits = "bits"
-    generate_and_write_rsa_keypair(filepath=fn_bits, password="", bits=bits)
+    _generate_and_write_rsa_keypair(filepath=fn_bits, bits=bits)
 
     priv = import_rsa_privatekey_from_file(fn_bits)
     # NOTE: Parse PEM with pyca/cryptography to get the key size property
@@ -158,10 +159,10 @@ class TestInterfaceFunctions(unittest.TestCase):
     fn_prompt = "prompt"
 
     # ... a passed pw ...
-    generate_and_write_rsa_keypair(filepath=fn_encrypted, password=pw)
+    _generate_and_write_rsa_keypair(filepath=fn_encrypted, password=pw)
     with mock.patch("securesystemslib.interface.get_password", return_value=pw):
       # ... and a prompted pw.
-      generate_and_write_rsa_keypair(filepath=fn_prompt)
+      _generate_and_write_rsa_keypair(filepath=fn_prompt, prompt=True)
 
       # Assert that both private keys are importable using the prompted pw ...
       import_rsa_privatekey_from_file(fn_prompt, prompt=True)
@@ -185,15 +186,30 @@ class TestInterfaceFunctions(unittest.TestCase):
 
 
     # TEST: Generation errors
+    for idx, (kwargs, err_msg) in enumerate([
+        # Error on empty password
+        ({"password": ""},
+          "encryption password must be 1 or more characters long"),
+        # Error on 'password' and 'prompt=True'
+        ({"password": pw, "prompt": True},
+          "passing 'password' and 'prompt=True' is not allowed")]):
+
+      with self.assertRaises(ValueError, msg="(row {})".format(idx)) as ctx:
+        _generate_and_write_rsa_keypair(**kwargs)
+
+      self.assertEqual(err_msg, str(ctx.exception),
+          "expected: '{}' got: '{}' (row {})".format(
+          err_msg, ctx.exception, idx))
 
     # Error on bad argument format
     for idx, kwargs in enumerate([
         {"bits": 1024}, # Too low
         {"bits": "not-an-int"},
         {"filepath": 123456}, # Not a string
-        {"password": 123456}]): # Not a string
+        {"password": 123456}, # Not a string
+        {"prompt": "not-a-bool"}]):
       with self.assertRaises(FormatError, msg="(row {})".format(idx)):
-        generate_and_write_rsa_keypair(**kwargs)
+        _generate_and_write_rsa_keypair(**kwargs)
 
 
     # TEST: Import errors
@@ -218,14 +234,14 @@ class TestInterfaceFunctions(unittest.TestCase):
         ([fn_encrypted], {}, CryptoError,
           "Password was not given but private key is encrypted"),
         # Error on encrypted but empty pw passed
-        ([fn_encrypted], {"password": ""}, ValueError,
-          "Password must be 1 or more character"),
+        ([fn_encrypted], {"password": ""}, CryptoError,
+          "Password was not given but private key is encrypted"),
         # Error on encrypted but bad pw passed
         ([fn_encrypted], {"password": "bad pw"}, CryptoError,
           "Bad decrypt. Incorrect password?"),
         # Error on pw and prompt
         ([fn_default], {"password": pw, "prompt": True}, ValueError,
-          "Passing 'password' and 'prompt' True is not allowed.")]):
+          "passing 'password' and 'prompt=True' is not allowed")]):
 
       with self.assertRaises(err, msg="(row {})".format(idx)) as ctx:
         import_rsa_privatekey_from_file(*args, **kwargs)
@@ -258,16 +274,19 @@ class TestInterfaceFunctions(unittest.TestCase):
     with self.assertRaises(FormatError):
       import_rsa_privatekey_from_file(fn_default, password=123456)
 
+    # bad prompt
+    with self.assertRaises(FormatError):
+      import_rsa_privatekey_from_file(fn_default, prompt="not-a-bool")
+
 
 
   def test_ed25519(self):
-    """Test ed25519 key generation and import interface functions. """
+    """Test ed25519 key _generation and import interface functions. """
 
     # TEST: Generate default keys and import
     # Assert location and format
     fn_default = "default"
-    fn_default_ret = generate_and_write_ed25519_keypair(
-        filepath=fn_default, password="")
+    fn_default_ret = _generate_and_write_ed25519_keypair(filepath=fn_default)
 
     pub = import_ed25519_publickey_from_file(fn_default + ".pub")
     priv = import_ed25519_privatekey_from_file(fn_default)
@@ -284,14 +303,14 @@ class TestInterfaceFunctions(unittest.TestCase):
     # Assert importable with empty prompt password and without password
     fn_empty_prompt = "empty_prompt"
     with mock.patch("securesystemslib.interface.get_password", return_value=""):
-      generate_and_write_ed25519_keypair(filepath=fn_empty_prompt)
+      _generate_and_write_ed25519_keypair(filepath=fn_empty_prompt)
       import_ed25519_privatekey_from_file(fn_empty_prompt, prompt=True)
     import_ed25519_privatekey_from_file(fn_empty_prompt)
 
 
     # TEST: Generate keys with auto-filename, i.e. keyid
     # Assert filename is keyid
-    fn_keyid = generate_and_write_ed25519_keypair(password="")
+    fn_keyid = _generate_and_write_ed25519_keypair()
     pub = import_ed25519_publickey_from_file(fn_keyid + ".pub")
     priv = import_ed25519_privatekey_from_file(fn_keyid)
     self.assertTrue(
@@ -303,10 +322,10 @@ class TestInterfaceFunctions(unittest.TestCase):
     fn_encrypted = "encrypted"
     fn_prompt = "prompt"
     # ... a passed pw ...
-    generate_and_write_ed25519_keypair(filepath=fn_encrypted, password=pw)
+    _generate_and_write_ed25519_keypair(filepath=fn_encrypted, password=pw)
     with mock.patch("securesystemslib.interface.get_password", return_value=pw):
       # ... and a prompted pw.
-      generate_and_write_ed25519_keypair(filepath=fn_prompt)
+      _generate_and_write_ed25519_keypair(filepath=fn_prompt, prompt=True)
 
       # Assert that both private keys are importable using the prompted pw ...
       import_ed25519_privatekey_from_file(fn_prompt, prompt=True)
@@ -343,13 +362,28 @@ class TestInterfaceFunctions(unittest.TestCase):
 
 
     # TEST: Generation errors
+    for idx, (kwargs, err_msg) in enumerate([
+        # Error on empty password
+        ({"password": ""},
+          "encryption password must be 1 or more characters long"),
+        # Error on 'password' and 'prompt=True'
+        ({"password": pw, "prompt": True},
+          "passing 'password' and 'prompt=True' is not allowed")]):
+
+      with self.assertRaises(ValueError, msg="(row {})".format(idx)) as ctx:
+        _generate_and_write_ed25519_keypair(**kwargs)
+
+      self.assertEqual(err_msg, str(ctx.exception),
+          "expected: '{}' got: '{}' (row {})".format(
+          err_msg, ctx.exception, idx))
 
     # Error on bad argument format
     for idx, kwargs in enumerate([
         {"filepath": 123456}, # Not a string
-        {"password": 123456}]): # Not a string
+        {"password": 123456}, # Not a string
+        {"prompt": "not-a-bool"}]):
       with self.assertRaises(FormatError, msg="(row {})".format(idx)):
-        generate_and_write_ed25519_keypair(**kwargs)
+        _generate_and_write_ed25519_keypair(**kwargs)
 
 
     # TEST: Import errors
@@ -381,15 +415,15 @@ class TestInterfaceFunctions(unittest.TestCase):
         ([fn_encrypted], {}, CryptoError,
           "Malformed Ed25519 key JSON, possibly due to encryption, "
           "but no password provided?"),
-        # Error on encrypted but empty pw passed
-        ([fn_encrypted], {"password": ""}, ValueError,
-          "Password must be 1 or more character"),
+        # Error on encrypted but empty pw
+        ([fn_encrypted], {"password": ""}, CryptoError,
+          "Decryption failed."),
         # Error on encrypted but bad pw passed
         ([fn_encrypted], {"password": "bad pw"}, CryptoError,
           "Decryption failed."),
         # Error on pw and prompt
         ([fn_default], {"password": pw, "prompt": True}, ValueError,
-          "Passing 'password' and 'prompt' True is not allowed.")]):
+          "passing 'password' and 'prompt=True' is not allowed")]):
 
       with self.assertRaises(err, msg="(row {})".format(idx)) as ctx:
         import_ed25519_privatekey_from_file(*args, **kwargs)
@@ -420,23 +454,20 @@ class TestInterfaceFunctions(unittest.TestCase):
     with self.assertRaises(FormatError):
       import_ed25519_privatekey_from_file(fn_default, password=123456)
 
+    # Error on bad prompt format
+    with self.assertRaises(FormatError):
+      import_ed25519_privatekey_from_file(fn_default, prompt="not-a-bool")
 
 
   def test_ecdsa(self):
-    """Test ecdsa key generation and import interface functions. """
-    # NOTE: Unlike rsa and ed25519, the ecdsa (key creation and private key
-    # import) interface only supports encrypted keys, even if the passed or
-    # prompted password is an empty string.
-
-    # TEST: Generate pw encrypted keys and import
+    """Test ecdsa key _generation and import interface functions. """
+    # TEST: Generate default keys and import
     # Assert location and format
-    pw = "pw"
     fn_default = "default"
-    fn_default_ret = generate_and_write_ecdsa_keypair(
-        filepath=fn_default, password=pw)
+    fn_default_ret = _generate_and_write_ecdsa_keypair(filepath=fn_default)
 
     pub = import_ecdsa_publickey_from_file(fn_default + ".pub")
-    priv = import_ecdsa_privatekey_from_file(fn_default, password=pw)
+    priv = import_ecdsa_privatekey_from_file(fn_default)
 
     self.assertEqual(fn_default, fn_default_ret)
     self.assertTrue(ECDSAKEY_SCHEMA.matches(pub))
@@ -445,13 +476,20 @@ class TestInterfaceFunctions(unittest.TestCase):
     # NOTE: There is no private key schema, at least check it has a value
     self.assertTrue(priv["keyval"]["private"])
 
+    # TEST: Generate unencrypted keys with empty prompt
+    # Assert importable with empty prompt password and without password
+    fn_empty_prompt = "empty_prompt"
+    with mock.patch("securesystemslib.interface.get_password", return_value=""):
+      _generate_and_write_ecdsa_keypair(filepath=fn_empty_prompt)
+      import_ecdsa_privatekey_from_file(fn_empty_prompt, prompt=True)
+    import_ecdsa_privatekey_from_file(fn_empty_prompt)
+
 
     # TEST: Generate keys with auto-filename, i.e. keyid
     # Assert filename is keyid
-    fn_keyid = generate_and_write_ecdsa_keypair(password=pw)
+    fn_keyid = _generate_and_write_ecdsa_keypair()
     pub = import_ecdsa_publickey_from_file(fn_keyid + ".pub")
-    priv = import_ecdsa_privatekey_from_file(
-        fn_keyid, password=pw)
+    priv = import_ecdsa_privatekey_from_file(fn_keyid)
     self.assertTrue(
         os.path.basename(fn_keyid) == pub["keyid"] == priv["keyid"])
 
@@ -461,14 +499,14 @@ class TestInterfaceFunctions(unittest.TestCase):
     fn_encrypted = "encrypted"
     fn_prompt = "prompt"
     # ...  a passed pw ...
-    generate_and_write_ecdsa_keypair(filepath=fn_encrypted, password=pw)
+    _generate_and_write_ecdsa_keypair(filepath=fn_encrypted, password=pw)
     with mock.patch("securesystemslib.interface.get_password", return_value=pw):
       # ... and a prompted pw.
-      generate_and_write_ecdsa_keypair(filepath=fn_prompt)
+      _generate_and_write_ecdsa_keypair(filepath=fn_prompt, prompt=True)
 
       # Assert that both private keys are importable using the prompted pw ...
-      import_ecdsa_privatekey_from_file(fn_prompt)
-      import_ecdsa_privatekey_from_file(fn_encrypted)
+      import_ecdsa_privatekey_from_file(fn_prompt, prompt=True)
+      import_ecdsa_privatekey_from_file(fn_encrypted, prompt=True)
 
     # ... and the passed pw.
     import_ecdsa_privatekey_from_file(fn_prompt, password=pw)
@@ -494,21 +532,36 @@ class TestInterfaceFunctions(unittest.TestCase):
 
 
     # TEST: Generation errors
+    for idx, (kwargs, err_msg) in enumerate([
+        # Error on empty password
+        ({"password": ""},
+          "encryption password must be 1 or more characters long"),
+        # Error on 'password' and 'prompt=True'
+        ({"password": pw, "prompt": True},
+          "passing 'password' and 'prompt=True' is not allowed")]):
+
+      with self.assertRaises(ValueError, msg="(row {})".format(idx)) as ctx:
+        _generate_and_write_ecdsa_keypair(**kwargs)
+
+      self.assertEqual(err_msg, str(ctx.exception),
+          "expected: '{}' got: '{}' (row {})".format(
+          err_msg, ctx.exception, idx))
 
     # Error on bad argument format
     for idx, kwargs in enumerate([
         {"filepath": 123456}, # Not a string
-        {"password": 123456}]): # Not a string
+        {"password": 123456}, # Not a string
+        {"prompt": "not-a-bool"}]):
       with self.assertRaises(FormatError, msg="(row {})".format(idx)):
-        generate_and_write_ecdsa_keypair(**kwargs)
+        _generate_and_write_ecdsa_keypair(**kwargs)
 
 
     # TEST: Import errors
 
     # Error on public key import...
     for idx, (fn, err_msg) in enumerate([
-        # Error on invalid custom json key format
-        (fn_default, "Cannot deserialize to a Python object"),
+        # Error on invalid json (custom key format)
+        (fn_encrypted, "Cannot deserialize to a Python object"),
         # Error on invalid custom key format
         (self.path_no_key, "Missing key")]):
       with self.assertRaises(Error, msg="(row {})".format(idx)) as ctx:
@@ -522,14 +575,23 @@ class TestInterfaceFunctions(unittest.TestCase):
     # Error on private key import...
     for idx, (args, kwargs, err, err_msg) in enumerate([
         # Error on not an ecdsa private key
-        ([self.path_ed25519], {"password": "password"}, FormatError,
-          "Invalid key type loaded"),
-        # Error on encrypted but empty pw passed
-        ([fn_default], {"password": ""}, CryptoError,
+        ([self.path_ed25519], {}, Error,
+          "Cannot deserialize to a Python object"),
+        # Error on not encrypted
+        ([fn_default], {"password": pw}, CryptoError,
+          "Invalid encrypted file."),
+        # Error on encrypted but no pw
+        ([fn_encrypted], {}, Error,
+          "Cannot deserialize to a Python object"),
+        # Error on encrypted but empty pw
+        ([fn_encrypted], {"password": ""}, CryptoError,
           "Decryption failed."),
-        # Error on encrypted but wrong pw passed
-        ([fn_default], {"password": "bad pw"}, CryptoError,
-          "Decryption failed.")]):
+        # Error on encrypted but bad pw passed
+        ([fn_encrypted], {"password": "bad pw"}, CryptoError,
+          "Decryption failed."),
+        # Error on pw and prompt
+        ([fn_default], {"password": pw, "prompt": True}, ValueError,
+          "passing 'password' and 'prompt=True' is not allowed")]):
 
       with self.assertRaises(err, msg="(row {})".format(idx)) as ctx:
         import_ecdsa_privatekey_from_file(*args, **kwargs)
@@ -538,6 +600,16 @@ class TestInterfaceFunctions(unittest.TestCase):
           "expected: '{}' got: '{}' (row {})".format(
           err_msg, ctx.exception, idx))
 
+    # Error on encrypted but bad pw prompted
+    err_msg = ("Decryption failed")
+    with self.assertRaises(CryptoError) as ctx, mock.patch(
+        "securesystemslib.interface.get_password", return_value="bad_pw"):
+      import_ecdsa_privatekey_from_file(fn_encrypted, prompt=True)
+
+    self.assertTrue(err_msg in str(ctx.exception),
+        "expected: '{}' got: '{}'".format(err_msg, ctx.exception))
+
+
     # Error on bad path format
     with self.assertRaises(FormatError):
       import_ecdsa_publickey_from_file(123456)
@@ -548,6 +620,74 @@ class TestInterfaceFunctions(unittest.TestCase):
     with self.assertRaises(FormatError): # bad password
       import_ecdsa_privatekey_from_file(fn_default, password=123456)
 
+    # Error on bad prompt format
+    with self.assertRaises(FormatError):
+      import_ecdsa_privatekey_from_file(fn_default, prompt="not-a-bool")
+
+
+
+  def test_generate_keypair_wrappers(self):
+    """Basic tests for thin wrappers around _generate_and_write_*_keypair.
+    See 'test_rsa', 'test_ed25519' and 'test_ecdsa' for more thorough key
+    generation tests for each key type.
+
+    """
+    key_pw = "pw"
+    for idx, (gen, gen_prompt, gen_plain, import_priv, schema) in enumerate([
+        (
+          generate_and_write_rsa_keypair,
+          generate_and_write_rsa_keypair_with_prompt,
+          generate_and_write_unencrypted_rsa_keypair,
+          import_rsa_privatekey_from_file,
+          RSAKEY_SCHEMA
+        ),
+        (
+          generate_and_write_ed25519_keypair,
+          generate_and_write_ed25519_keypair_with_prompt,
+          generate_and_write_unencrypted_ed25519_keypair,
+          import_ed25519_privatekey_from_file,
+          ED25519KEY_SCHEMA
+        ),
+        (
+          generate_and_write_ecdsa_keypair,
+          generate_and_write_ecdsa_keypair_with_prompt,
+          generate_and_write_unencrypted_ecdsa_keypair,
+          import_ecdsa_privatekey_from_file,
+          ECDSAKEY_SCHEMA)]):
+
+      assert_msg = "(row {})".format(idx)
+      # Test generate_and_write_*_keypair creates an encrypted private key
+      fn_encrypted = gen(key_pw)
+      priv = import_priv(fn_encrypted, key_pw)
+      self.assertTrue(schema.matches(priv), assert_msg)
+
+      # Test generate_and_write_*_keypair errors if password is None or empty
+      with self.assertRaises(FormatError, msg=assert_msg):
+        fn_encrypted = gen(None)
+      with self.assertRaises(ValueError, msg=assert_msg):
+        fn_encrypted = gen("")
+
+      # Test generate_and_write_*_keypair_with_prompt creates encrypted private
+      # key
+      with mock.patch(
+          "securesystemslib.interface.get_password", return_value=key_pw):
+        fn_prompt = gen_prompt()
+      priv = import_priv(fn_prompt, key_pw)
+      self.assertTrue(schema.matches(priv), assert_msg)
+
+      # Test generate_and_write_*_keypair_with_prompt creates unencrypted
+      # private key if no password is entered
+      with mock.patch(
+          "securesystemslib.interface.get_password", return_value=""):
+        fn_empty_prompt = gen_prompt()
+      priv = import_priv(fn_empty_prompt)
+      self.assertTrue(schema.matches(priv), assert_msg)
+
+      # Test generate_and_write_unencrypted_*_keypair doesn't encrypt
+      fn_unencrypted = gen_plain()
+      priv = import_priv(fn_unencrypted)
+      self.assertTrue(schema.matches(priv), assert_msg)
+
 
 
   def test_import_publickeys_from_file(self):
@@ -593,6 +733,37 @@ class TestInterfaceFunctions(unittest.TestCase):
           [KEY_TYPE_ED25519])
 
 
+  def test_import_privatekey_from_file(self):
+    """Test generic private key import function. """
+
+    pw = "password"
+    for idx, (path, key_type, key_schema) in enumerate([
+        (self.path_rsa, None, RSAKEY_SCHEMA), # default key type
+        (self.path_rsa, KEY_TYPE_RSA, RSAKEY_SCHEMA),
+        (self.path_ed25519, KEY_TYPE_ED25519, ED25519KEY_SCHEMA),
+        (self.path_ecdsa, KEY_TYPE_ECDSA, ECDSAKEY_SCHEMA)]):
+
+      # Successfully import key per supported type, with ...
+      # ... passed password
+      key = import_privatekey_from_file(path, key_type=key_type, password=pw)
+      self.assertTrue(key_schema.matches(key), "(row {})".format(idx))
+
+      # ... entered password on mock-prompt
+      with mock.patch("securesystemslib.interface.get_password", return_value=pw):
+        key = import_privatekey_from_file(path, key_type=key_type, prompt=True)
+      self.assertTrue(key_schema.matches(key), "(row {})".format(idx))
+
+    # Error on wrong key for default key type
+    with self.assertRaises(Error):
+      import_privatekey_from_file(self.path_ed25519, password=pw)
+
+    # Error on unsupported key type
+    with self.assertRaises(FormatError):
+      import_privatekey_from_file(
+          self.path_rsa, key_type="KEY_TYPE_UNSUPPORTED", password=pw)
+
+
+
 # Run the test cases.
 if __name__ == '__main__':
   unittest.main()
